# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: servicelayer/rpc/ocr.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from servicelayer.rpc import common_pb2 as servicelayer_dot_rpc_dot_common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='servicelayer/rpc/ocr.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x1aservicelayer/rpc/ocr.proto\x1a\x1dservicelayer/rpc/common.proto\"y\n\x05Image\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x12\x11\n\tlanguages\x18\x02 \x03(\t\x12\x19\n\x04mode\x18\x03 \x01(\x0e\x32\x0b.Image.Mode\"4\n\x04Mode\x12\t\n\x05OTHER\x10\x00\x12\x08\n\x04PAGE\x10\x01\x12\x08\n\x04WORD\x10\x02\x12\r\n\tCHARACTER\x10\x03\x32+\n\rRecognizeText\x12\x1a\n\tRecognize\x12\x06.Image\x1a\x05.Textb\x06proto3')
  ,
  dependencies=[servicelayer_dot_rpc_dot_common__pb2.DESCRIPTOR,])



_IMAGE_MODE = _descriptor.EnumDescriptor(
  name='Mode',
  full_name='Image.Mode',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='OTHER', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PAGE', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='WORD', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CHARACTER', index=3, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=130,
  serialized_end=182,
)
_sym_db.RegisterEnumDescriptor(_IMAGE_MODE)


_IMAGE = _descriptor.Descriptor(
  name='Image',
  full_name='Image',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='data', full_name='Image.data', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='languages', full_name='Image.languages', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='mode', full_name='Image.mode', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _IMAGE_MODE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=61,
  serialized_end=182,
)

_IMAGE.fields_by_name['mode'].enum_type = _IMAGE_MODE
_IMAGE_MODE.containing_type = _IMAGE
DESCRIPTOR.message_types_by_name['Image'] = _IMAGE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Image = _reflection.GeneratedProtocolMessageType('Image', (_message.Message,), dict(
  DESCRIPTOR = _IMAGE,
  __module__ = 'servicelayer.rpc.ocr_pb2'
  # @@protoc_insertion_point(class_scope:Image)
  ))
_sym_db.RegisterMessage(Image)



_RECOGNIZETEXT = _descriptor.ServiceDescriptor(
  name='RecognizeText',
  full_name='RecognizeText',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=184,
  serialized_end=227,
  methods=[
  _descriptor.MethodDescriptor(
    name='Recognize',
    full_name='RecognizeText.Recognize',
    index=0,
    containing_service=None,
    input_type=_IMAGE,
    output_type=servicelayer_dot_rpc_dot_common__pb2._TEXT,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_RECOGNIZETEXT)

DESCRIPTOR.services_by_name['RecognizeText'] = _RECOGNIZETEXT

# @@protoc_insertion_point(module_scope)



import time
import random
import logging

log = logging.getLogger(__name__)


def backoff(failures=0):
    """Implement a random, growing delay between external service retries."""
    sleep = max(1, failures - 1) + random.random()
    log.debug("Back-off: %.2fs", sleep)
    time.sleep(sleep)


def service_retries():
    """A default number of tries to re-try an external service."""
    return range(30)



from unittest import TestCase

from servicelayer.cache import get_fakeredis
from servicelayer.process import ServiceQueue, RateLimit, Progress


class ProcessTest(TestCase):

    def test_service_queue(self):
        conn = get_fakeredis()
        ds = 'test_1'
        queue = ServiceQueue(conn, ServiceQueue.OP_INGEST, ds)
        status = queue.progress.get()
        assert status['pending'] == 0
        assert status['finished'] == 0
        assert queue.is_done()
        queue.queue_task({'test': 'foo'}, {})
        status = queue.progress.get()
        assert status['pending'] == 1
        assert status['finished'] == 0
        assert not queue.is_done()
        task = ServiceQueue.get_operation_task(conn, ServiceQueue.OP_INGEST,
                                               timeout=None)
        nq, payload, context = task
        assert nq.dataset == queue.dataset
        assert payload['test'] == 'foo'
        status = queue.progress.get()
        assert status['pending'] == 1
        assert status['finished'] == 0
        queue.task_done()
        status = queue.progress.get()
        assert status['pending'] == 0
        assert status['finished'] == 1
        task = ServiceQueue.get_operation_task(conn, ServiceQueue.OP_INGEST,
                                               timeout=1)
        nq, payload, context = task
        assert payload is None
        queue.remove()
        status = queue.progress.get()
        assert status['pending'] == 0
        assert status['finished'] == 0

    def test_queue_clear(self):
        conn = get_fakeredis()
        ds = 'test_1'
        queue = ServiceQueue(conn, ServiceQueue.OP_INGEST, ds)
        queue.queue_task({'test': 'foo'}, {})
        status = queue.progress.get()
        assert status['pending'] == 1
        ServiceQueue.remove_dataset(conn, ds)
        status = queue.progress.get()
        assert status['pending'] == 0

    def test_progress(self):
        conn = get_fakeredis()
        ds = 'test_2'
        progress = Progress(conn, ServiceQueue.OP_INGEST, ds)
        status = progress.get()
        assert status['pending'] == 0
        assert status['finished'] == 0
        progress.mark_pending()
        status = progress.get()
        assert status['pending'] == 1
        assert status['finished'] == 0
        full = Progress.get_dataset_status(conn, ds)
        assert full['pending'] == 1
        progress.mark_finished()
        status = progress.get()
        assert status['pending'] == 0
        assert status['finished'] == 1
        progress.remove()
        status = progress.get()
        assert status['finished'] == 0

    def test_rate(self):
        conn = get_fakeredis()
        limit = RateLimit(conn, 'banana', limit=10)
        assert limit.check()
        limit.update()
        assert limit.check()
        for i in range(13):
            limit.update()
        assert not limit.check()



import json
import time
import random
import logging
from banal import ensure_list
from redis.exceptions import BusyLoadingError

from servicelayer.settings import REDIS_LONG
from servicelayer.cache import make_key

log = logging.getLogger(__name__)
PREFIX = 'sla'


class ServiceQueue(object):
    OP_INGEST = 'ingest'
    OP_INDEX = 'index'
    OP_ANALYSIS = 'analysis'

    PRIO_HIGH = 3
    PRIO_MEDIUM = 2
    PRIO_LOW = 1
    PRIORITIES = [PRIO_HIGH, PRIO_MEDIUM, PRIO_LOW]

    def __init__(self, conn, operation, dataset, priority=PRIO_MEDIUM):
        self.conn = conn
        self.operation = operation
        self.dataset = dataset
        self.priority = priority
        self.progress = Progress(conn, operation, dataset)
        self.queue_key = make_key(PREFIX, 'q', operation, dataset)
        self.ops_queue_key = make_key(PREFIX, 'qo', operation, priority)

    def queue_task(self, payload, context):
        self.conn.sadd(self.ops_queue_key, self.queue_key)
        data = json.dumps({
            'context': context or {},
            'payload': payload,
            'dataset': self.dataset,
            'operation': self.operation,
            'priority': self.priority
        })
        self.conn.rpush(self.queue_key, data)
        self.progress.mark_pending()

    def task_done(self):
        self.progress.mark_finished()

    def is_done(self):
        status = self.progress.get()
        return status.get('pending') < 1

    def remove(self):
        self.conn.sadd(self.ops_queue_key, self.queue_key)
        self.conn.delete(self.queue_key)
        self.progress.remove()

    @classmethod
    def _get_operation_queues(cls, conn, operations):
        """Return all the active queues for the given operation."""
        queues = []
        for priority in cls.PRIORITIES:
            prio_queues = []
            for op in ensure_list(operations):
                key = make_key(PREFIX, 'qo', op, priority)
                prio_queues.extend(conn.smembers(key))
            # TODO: do we want to random.shuffle?
            random.shuffle(prio_queues)
            for queue in prio_queues:
                if queue not in queues:
                    queues.append(queue)
        return queues

    @classmethod
    def get_operation_task(cls, conn, operations, timeout=0):
        """Retrieve a single task from the highest-priority queue that has
        work pending."""
        try:
            queues = cls._get_operation_queues(conn, operations)
            if not len(queues):
                return (None, None, None)
            # Support a magic value to not block, i.e. timeout None
            if timeout is None:
                # LPOP does not support multiple lists.
                for queue in queues:
                    task_data = conn.lpop(queue)
                    if task_data is not None:
                        break
            else:
                task_data = conn.blpop(queues, timeout=timeout)
                if task_data is not None:
                    key, task_data = task_data

            if task_data is None:
                return (None, None, None)

            task = json.loads(task_data)
            operation = task.get('operation')
            dataset = task.get('dataset')
            priority = task.get('priority')
            queue = cls(conn, operation, dataset, priority=priority)
            return (queue, task.get('payload'), task.get('context'))
        except BusyLoadingError:
            time.sleep(timeout + 1)
            return (None, None, None)

    @classmethod
    def remove_dataset(cls, conn, dataset):
        """Delete all known queues associated with a dataset."""
        for operation in Progress.get_dataset_operations(conn, dataset):
            for priority in cls.PRIORITIES:
                queue = cls(conn, operation, dataset, priority=priority)
                queue.remove()


class Progress(object):
    """An abstraction for the notion of a process that covers a set
    of items. This is simplified and does not cover the notion of a
    task being in-progress."""
    PENDING = 'pending'
    FINISHED = 'finished'
    # TODO: do we need a 'running' state?

    def __init__(self, conn, operation, dataset):
        _key = make_key(PREFIX, 'p', operation, dataset)
        self.conn = conn
        self.operation = operation
        self.dataset = dataset
        self.dataset_key = make_key(PREFIX, 'qd', dataset)
        self.pending_key = make_key(_key, self.PENDING)
        self.finished_key = make_key(_key, self.FINISHED)

    def mark_active(self):
        """Add the dataset to the list of datasets that are
        running."""
        self.conn.sadd(self.dataset_key, self.operation)

    def mark_pending(self, amount=1):
        self.mark_active()
        self.conn.incr(self.pending_key, amount=amount)

    def mark_finished(self, amount=1):
        """Move items from the pending to the finished set."""
        self.conn.decr(self.pending_key, amount=amount)
        self.conn.incr(self.finished_key, amount=amount)

    def put_finished(self, amount=1):
        """Add a number of items to the finished set without reducing
        the count of pending items."""
        self.mark_active()
        self.conn.incr(self.finished_key, amount=amount)

    def remove(self):
        self.conn.srem(self.dataset_key, self.operation)
        self.conn.expire(self.dataset_key, REDIS_LONG)
        self.conn.delete(self.pending_key, self.finished_key)

    def get(self):
        """Get the current status."""
        keys = (self.pending_key, self.finished_key)
        pending, finished = self.conn.mget(keys)
        return {
            'operation': self.operation,
            'pending': max(0, int(pending or 0)),
            'finished': max(0, int(finished or 0)),
        }

    @classmethod
    def get_dataset_operations(cls, conn, dataset):
        return conn.smembers(make_key(PREFIX, 'qd', dataset))

    @classmethod
    def get_dataset_status(cls, conn, dataset):
        """Aggregate status for all operations on the given dataset."""
        status = {'finished': 0, 'pending': 0, 'operations': []}
        for operation in cls.get_dataset_operations(conn, dataset):
            progress = cls(conn, operation, dataset).get()
            status['operations'].append(progress)
            status['finished'] += progress['finished']
            status['pending'] += progress['pending']
        return status


class RateLimit(object):
    """Limit the rate of operations on a given resource during a
    stated interval."""

    def __init__(self, conn, resource, limit=100, interval=60, unit=1):
        self.conn = conn
        self.resource = resource
        self.limit = max(0.1, limit)
        self.interval = max(1, interval)
        self.unit = unit

    def _time(self):
        return int(time.time() / self.unit)

    def _keys(self):
        base = self._time()
        for slot in range(base, base + self.interval):
            yield make_key(PREFIX, 'rate', self.resource, slot)

    def update(self, amount=1):
        """Set the cached counts for stats keeping."""
        pipe = self.conn.pipeline()
        for key in self._keys():
            pipe.incr(key, amount=amount)
            pipe.expire(key, (self.interval * self.unit) + 2)
        values = pipe.execute()[::2]
        return (sum(values) / self.interval)

    def check(self):
        """Check if the resource has exceeded the rate limit."""
        key = make_key(PREFIX, 'rate', self.resource, self._time())
        count = int(self.conn.get(key) or 0)
        return count < self.limit

    def comply(self, amount=1):
        """Update, then sleep for the time required to adhere to the
        rate limit."""
        # FIXME: this is trying to be smart, which will probably
        # backfire. Rather than just halt when the rate is exceeded,
        # this is trying to average down calls to the appropriate
        # frequency.
        rate = self.update(amount=amount) / self.limit
        expected = self.interval / self.limit
        excess = rate - expected
        if excess > 0:
            time.sleep(excess)
        return rate



import logging
import threading
from banal import ensure_list
from grpc import RpcError, StatusCode, insecure_channel

from servicelayer import settings
from servicelayer.rpc.ocr_pb2 import Image
# from servicelayer.rpc.common_pb2 import Text
from servicelayer.rpc.ocr_pb2_grpc import RecognizeTextStub
# from servicelayer.rpc.entityextract_pb2 import ExtractedEntity  # noqa
# from servicelayer.rpc.entityextract_pb2_grpc import EntityExtractStub
from servicelayer.util import backoff, service_retries

log = logging.getLogger(__name__)
TEMP_ERRORS = (StatusCode.CANCELLED,
               StatusCode.UNAVAILABLE,
               StatusCode.RESOURCE_EXHAUSTED)


class RpcMixin(object):
    """Helper mixing to manage the connectivity with a gRPC endpoint."""
    SERVICE = None

    def has_channel(self):
        return self.SERVICE is not None

    @property
    def channel(self):
        """Lazily connect to the RPC service."""
        if not self.has_channel():
            return
        if not hasattr(self, 'tl') or self.tl is None:
            self.tl = threading.local()
        if not hasattr(self.tl, 'channel') or self.tl.channel is None:
            options = (
                # ('grpc.keepalive_time_ms', settings.GRPC_CONN_AGE),
                ('grpc.keepalive_timeout_ms', settings.GRPC_CONN_AGE),
                ('grpc.max_connection_age_ms', settings.GRPC_CONN_AGE),
                ('grpc.max_connection_idle_ms', settings.GRPC_CONN_AGE),
                ('grpc.lb_policy_name', settings.GRPC_LB_POLICY)
            )
            self.tl.channel = insecure_channel(self.SERVICE, options)
        return self.tl.channel

    def reset_channel(self):
        try:
            self.tl.channel.close()
        except Exception:
            log.warning("Cannot close gRPC connection")
        self.tl.channel = None


class TextRecognizerService(RpcMixin):
    SERVICE = settings.OCR_SERVICE

    def Recognize(self, data, languages=None):
        if not self.has_channel():
            log.warning("gRPC: OCR not configured.")
            return

        for attempt in service_retries():
            try:
                service = RecognizeTextStub(self.channel)
                languages = ensure_list(languages)
                image = Image(data=data, languages=languages)
                return service.Recognize(image)
            except RpcError as e:
                log.warning("gRPC [%s]: %s", e.code(), e.details())
                self.reset_channel()
                if e.code() not in TEMP_ERRORS:
                    return
                backoff(failures=attempt)


# class EntityExtractService(RpcMixin):
#     SERVICE = settings.NER_SERVICE

#     def Extract(self, text, languages):
#         if not self.has_channel():
#             log.warning("gRPC: entity extraction not configured.")
#             return

#         for attempt in service_retries():
#             try:
#                 service = EntityExtractStub(self.channel)
#                 req = Text(text=text, languages=languages)
#                 yield from service.Extract(req)
#                 return
#             except RpcError as e:
#                 log.warning("gRPC [%s]: %s", e.code(), e.details())
#                 if e.code() not in TEMP_ERRORS:
#                     return
#                 self.reset_channel()
#                 backoff(failures=attempt)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from servicelayer.rpc import common_pb2 as servicelayer_dot_rpc_dot_common__pb2
from servicelayer.rpc import entityextract_pb2 as servicelayer_dot_rpc_dot_entityextract__pb2


class EntityExtractStub(object):
  """The EntityService provides a simple extractor for NLP services that
  accept a stream of text objects (with language metadata) and return
  a set of entities extracted from the source text.
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.Extract = channel.unary_stream(
        '/EntityExtract/Extract',
        request_serializer=servicelayer_dot_rpc_dot_common__pb2.Text.SerializeToString,
        response_deserializer=servicelayer_dot_rpc_dot_entityextract__pb2.ExtractedEntity.FromString,
        )


class EntityExtractServicer(object):
  """The EntityService provides a simple extractor for NLP services that
  accept a stream of text objects (with language metadata) and return
  a set of entities extracted from the source text.
  """

  def Extract(self, request, context):
    """Extract entities from the given text.
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_EntityExtractServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'Extract': grpc.unary_stream_rpc_method_handler(
          servicer.Extract,
          request_deserializer=servicelayer_dot_rpc_dot_common__pb2.Text.FromString,
          response_serializer=servicelayer_dot_rpc_dot_entityextract__pb2.ExtractedEntity.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'EntityExtract', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: servicelayer/rpc/entityextract.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from servicelayer.rpc import common_pb2 as servicelayer_dot_rpc_dot_common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='servicelayer/rpc/entityextract.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n$servicelayer/rpc/entityextract.proto\x1a\x1dservicelayer/rpc/common.proto\"\xf7\x01\n\x0f\x45xtractedEntity\x12\x0c\n\x04text\x18\x01 \x01(\t\x12#\n\x04type\x18\x02 \x01(\x0e\x32\x15.ExtractedEntity.Type\x12\r\n\x05start\x18\x03 \x01(\x04\x12\x0b\n\x03\x65nd\x18\x04 \x01(\x04\"\x94\x01\n\x04Type\x12\t\n\x05OTHER\x10\x00\x12\n\n\x06PERSON\x10\x01\x12\x10\n\x0cORGANIZATION\x10\x02\x12\x0b\n\x07\x43OMPANY\x10\x03\x12\x0c\n\x08LOCATION\x10\x04\x12\x0b\n\x07\x43OUNTRY\x10\x05\x12\x0c\n\x08LANGUAGE\x10\x06\x12\r\n\tIPADDRESS\x10\x07\x12\t\n\x05PHONE\x10\x08\x12\t\n\x05\x45MAIL\x10\t\x12\x08\n\x04IBAN\x10\n25\n\rEntityExtract\x12$\n\x07\x45xtract\x12\x05.Text\x1a\x10.ExtractedEntity0\x01\x62\x06proto3')
  ,
  dependencies=[servicelayer_dot_rpc_dot_common__pb2.DESCRIPTOR,])



_EXTRACTEDENTITY_TYPE = _descriptor.EnumDescriptor(
  name='Type',
  full_name='ExtractedEntity.Type',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='OTHER', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PERSON', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ORGANIZATION', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='COMPANY', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LOCATION', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='COUNTRY', index=5, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LANGUAGE', index=6, number=6,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='IPADDRESS', index=7, number=7,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PHONE', index=8, number=8,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='EMAIL', index=9, number=9,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='IBAN', index=10, number=10,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=171,
  serialized_end=319,
)
_sym_db.RegisterEnumDescriptor(_EXTRACTEDENTITY_TYPE)


_EXTRACTEDENTITY = _descriptor.Descriptor(
  name='ExtractedEntity',
  full_name='ExtractedEntity',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='text', full_name='ExtractedEntity.text', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='ExtractedEntity.type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='start', full_name='ExtractedEntity.start', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='end', full_name='ExtractedEntity.end', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _EXTRACTEDENTITY_TYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=72,
  serialized_end=319,
)

_EXTRACTEDENTITY.fields_by_name['type'].enum_type = _EXTRACTEDENTITY_TYPE
_EXTRACTEDENTITY_TYPE.containing_type = _EXTRACTEDENTITY
DESCRIPTOR.message_types_by_name['ExtractedEntity'] = _EXTRACTEDENTITY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ExtractedEntity = _reflection.GeneratedProtocolMessageType('ExtractedEntity', (_message.Message,), dict(
  DESCRIPTOR = _EXTRACTEDENTITY,
  __module__ = 'servicelayer.rpc.entityextract_pb2'
  # @@protoc_insertion_point(class_scope:ExtractedEntity)
  ))
_sym_db.RegisterMessage(ExtractedEntity)



_ENTITYEXTRACT = _descriptor.ServiceDescriptor(
  name='EntityExtract',
  full_name='EntityExtract',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=321,
  serialized_end=374,
  methods=[
  _descriptor.MethodDescriptor(
    name='Extract',
    full_name='EntityExtract.Extract',
    index=0,
    containing_service=None,
    input_type=servicelayer_dot_rpc_dot_common__pb2._TEXT,
    output_type=_EXTRACTEDENTITY,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_ENTITYEXTRACT)

DESCRIPTOR.services_by_name['EntityExtract'] = _ENTITYEXTRACT

# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from servicelayer.rpc import common_pb2 as servicelayer_dot_rpc_dot_common__pb2
from servicelayer.rpc import ocr_pb2 as servicelayer_dot_rpc_dot_ocr__pb2


class RecognizeTextStub(object):
  """Request OCR to be performed on the given image of a
  piece of text.
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.Recognize = channel.unary_unary(
        '/RecognizeText/Recognize',
        request_serializer=servicelayer_dot_rpc_dot_ocr__pb2.Image.SerializeToString,
        response_deserializer=servicelayer_dot_rpc_dot_common__pb2.Text.FromString,
        )


class RecognizeTextServicer(object):
  """Request OCR to be performed on the given image of a
  piece of text.
  """

  def Recognize(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_RecognizeTextServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'Recognize': grpc.unary_unary_rpc_method_handler(
          servicer.Recognize,
          request_deserializer=servicelayer_dot_rpc_dot_ocr__pb2.Image.FromString,
          response_serializer=servicelayer_dot_rpc_dot_common__pb2.Text.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'RecognizeText', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))
